# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""The By implementation."""


class By(object):
    """Set of supported locator strategies."""

    ID = "resource-id"
    TEXT = "text"
    CLASS = "class"
    CONTENT_DESC = "content-desc"
    PACKAGE_NAME = 'package'
    CHECKABLE = 'checkable'
    CHECKED = 'checked'
    CLICKABLE = 'clickable'
    ENABLED = 'enabled'
    FOCUSABLE = 'focusable'
    SCROLLABLE = 'scrollable'
    LONG_CLICKABLE = 'long-clickable'
    PASSWORD = 'password'
    SELECTED = 'selected'
    BOUNDS = 'bounds'
